﻿--------------------------------------------------------------------------------------------------------
--                                             AceAddon init                                          --
--------------------------------------------------------------------------------------------------------
Broker_TolBarad = LibStub("AceAddon-3.0"):NewAddon("Broker_TolBarad", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local L = LibStub("AceLocale-3.0"):GetLocale("Broker_TolBarad")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDB = LibStub("AceDB-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")

--------------------------------------------------------------------------------------------------------
--                                Broker_TolBarad variables and defaults                              --
--------------------------------------------------------------------------------------------------------
local FACTION_UNKNOWN = 0
local FACTION_ALLIANCE = 1
local FACTION_HORDE = 2
local FACTION_BATTLE = 3

local faction = FACTION_UNKNOWN
local nextBattle = nil

local profileDB
local DATABASE_DEFAULTS = {
	profile = {
		showSeconds = false,
		tellSelf = true,
		tellGuild = true,
		tellParty = false,
		tellRaid = false,
		timeNow = false,
		timeFive = true,
		timeFifteen = true,
		timeThirty = true,
	},
}

--------------------------------------------------------------------------------------------------------
--                                 Broker_TolBarad options panel                                      --
--------------------------------------------------------------------------------------------------------
Broker_TolBarad.options = {
	type = "group",
	name = "Broker Tol Barad",
	args = {
		general = {
			order = 1,
			type = "group",
			name = L["General Settings"],
			cmdInline = true,
			args = {
				seconds = {
					order = 1,
					type = "toggle",
					name = L["Show seconds"],
					desc = L["Toggle to show seconds."],
					get = function()
						return profileDB.showSeconds
					end,
					set = function(key, value)
						profileDB.showSeconds = value
					end,
				},
				announceTo = {
					order = 2,
					type = 'group',
					name = L["Announce to"],
					desc = L["Choose whom to announce when battle will start."],
					args = {
						self = {
							order = 1,
							type = 'toggle',
							name = L["Self"],
							desc = L["Warn self on upcoming battle."],
							get = function() return profileDB.tellSelf end,
							set = function(key, value) profileDB.tellSelf = value end,
						},
						guild = {
							order = 2,
							type = 'toggle',
							name = L["Guild"],
							desc = L["Warn guild on upcoming battle."],
							get = function() return profileDB.tellGuild end,
							set = function(key, value) profileDB.tellGuild = value end,
						},
						party = {
							order = 3,
							type = 'toggle',
							name = L["Party"],
							desc = L["Warn party on upcoming battle."],
							get = function() return profileDB.tellParty end,
							set = function(key, value) profileDB.tellParty = value end,
						},
						raid = {
							order = 4,
							type = 'toggle',
							name = L["Raid"],
							desc = L["Warn raid on upcoming battle."],
							get = function() return profileDB.tellRaid end,
							set = function(key, value) profileDB.tellRaid = value end,
						},
					},
				},
				announceTime = {
					order = 3,
					type = 'group',
					name = L["Announce time"],
					desc = L["Choose when to announce the upcoming battle."],
					args = {
						now= {
							order = 1,
							type = 'toggle',
							name = L["Start"],
							desc = L["Warn at battle start."],
							get = function() return profileDB.timeNow end,
							set = function(key, value) profileDB.timeNow = value end,
						},
						five= {
							order = 2,
							type = 'toggle',
							name = "5" .. L[" minutes."],
							desc = L["Warn "] .. "5" .. L[" minutes before battle."],
							get = function() return profileDB.timeFive end,
							set = function(key, value) profileDB.timeFive= value end,
						},
						fifteen= {
							order = 3,
							type = 'toggle',
							name = "15" .. L[" minutes."],
							desc = L["Warn "] .. "15" .. L[" minutes before battle."],
							get = function() return profileDB.timeFifteen end,
							set = function(key, value) profileDB.timeFifteen = value end,
						},
						thirty = {
							order = 4,
							type = 'toggle',
							name = "30" .. L[" minutes."],
							desc = L["Warn "] .. "30" .. L[" minutes before battle."],
							get = function() return profileDB.timeThirty end,
							set = function(key, value) profileDB.timeThirty = value end,
						},
					},
				},
			},
		},
	},
}

function Broker_TolBarad:SetupOptions()
	Broker_TolBarad.options.args.profile = AceDBOptions:GetOptionsTable(self.db)
	Broker_TolBarad.options.args.profile.order = -2

	AceConfig:RegisterOptionsTable("Broker_TolBarad", Broker_TolBarad.options, nil)
	
	self.optionsFrames = {}
	self.optionsFrames.general = AceConfigDialog:AddToBlizOptions("Broker_TolBarad", nil, nil, "general")
	self.optionsFrames.profile = AceConfigDialog:AddToBlizOptions("Broker_TolBarad", L["Profiles"], "Broker_TolBarad", "profile")
end

--------------------------------------------------------------------------------------------------------
--                                          Broker_TolBarad core                                      --
--------------------------------------------------------------------------------------------------------
function Broker_TolBarad:OnInitialize()
	self.db = AceDB:New("Broker_TolBaradDB", DATABASE_DEFAULTS, true)
	if not self.db then
		Print("Error: Database not loaded correctly.  Please exit out of WoW and delete Broker_TolBarad.lua found in: \\World of Warcraft\\WTF\\Account\\<Account Name>>\\SavedVariables\\")
	end
	
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	
	profileDB = self.db.profile
	self:SetupOptions()
end

function Broker_TolBarad:OnEnable()
	self:ScheduleRepeatingTimer("MainUpdate", 1)
end

-- LDB object
Broker_TolBarad.obj = ldb:NewDataObject("Broker_TolBarad", {
	type = "data source",
	label = "Broker_TolBarad",
	text = "",
	icon = "Interface\\Icons\\Achievement_zone_tolbarad",
	OnClick = function(frame, msg)
		if msg == "LeftButton" then
			Broker_TolBarad:OnClick()
		elseif msg == "RightButton" then
			Broker_TolBarad:ShowConfig()
		end
	end,
	OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
		tooltip:AddLine("Broker_TolBarad "..GetAddOnMetadata("Broker_TolBarad", "Version"))
		tooltip:AddLine(L["Shift-hint"])
		tooltip:AddLine(L["Right-hint"])
	end,
})

-- Main update function
function Broker_TolBarad:MainUpdate()
	if faction == FACTION_UNKNOWN or faction == FACTION_BATTLE then
		self:UpdateIcon()
	end
	if nextBattle == nil then
		self:UpdateTime()
	end
	local timeText
	if faction == FACTION_BATTLE then
		timeText = L["Battle"]
	elseif not nextBattle then
		timeText = L["Unknown"]
	elseif nextBattle then
		local seconds = nextBattle - time()
		if (seconds > 0) then
			timeText = self:GetTimeText(seconds)
			if (seconds > 3595) then
				faction = FACTION_UNKNOWN
			end
			if seconds == 1800 and profileDB.timeThirty then
				self:Warn(30)
			elseif seconds == 900 and profileDB.timeFifteen then
				self:Warn(15)
			elseif seconds == 300 and profileDB.timeFive then
				self:Warn(5)
			end
		elseif (seconds == 0 ) then
			if profileDB.timeNow then
				self:Warn(0)
			end
			faction = FACTION_UNKNOWN
			nextBattle = nextBattle + 3600
			timeText = self:GetTimeText(seconds)
		else
			faction = FACTION_UNKNOWN
			nextBattle = nil
			timeText = L["Unknown"]
		end
	end
	if timeText then
		self.obj.text = "|cffffff00" .. timeText .. "|r"
	end
end

--------------------------------------------------------------------------------------------------------
--                                       Broker_TolBarad functions                                    --
--------------------------------------------------------------------------------------------------------

-- Called after profile changed
function Broker_TolBarad:OnProfileChanged(event, database, newProfileKey)
	profileDB = database.profile
end

-- Open config window
function Broker_TolBarad:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.profile)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.general)
end

-- On click event handler
function Broker_TolBarad:OnClick()
	if IsShiftKeyDown() then
		if ChatFrame1EditBox:IsVisible() then
			local message = self:GetAnnounceText(faction, nextBattle)
			ChatFrame1EditBox:Insert(message)
		end
	end
end

-- Update time.
function Broker_TolBarad:UpdateTime()
	local pvpID, zone, isActive, canQueue, startTime, canEnter = GetWorldPVPAreaInfo(2);
	if startTime then 
		nextBattle = time() + startTime
	end
end

-- Search for TB status. Update faction and icon if needed.
function Broker_TolBarad:UpdateIcon()
	print("update")
	local pvpID, zone, isActive, canQueue, startTime, canEnter = GetWorldPVPAreaInfo(2);
	if not WorldMapFrame:IsVisible() and not isActive then
		local continentIndex = GetCurrentMapContinent()
		if not IsInInstance() then
			local zoneIndex = GetCurrentMapZone()
			SetMapZoom(2)
			local n = GetNumMapLandmarks()
			if n > 0 then
				for i = 1, n do
					local name, _, nr = GetMapLandmarkInfo(i)
					if name == L["Baradin Hold"] then
						if nr == 46 then
							self.obj.icon = "Interface\\Addons\\Broker_TolBarad\\alliance"
							faction = FACTION_ALLIANCE
							self:UpdateTime()
							return
						elseif nr == 48 then
							self.obj.icon = "Interface\\Addons\\Broker_TolBarad\\horde"
							faction = FACTION_HORDE
							self:UpdateTime()
							return
						elseif nr == 101 then
							self.obj.icon = "Interface\\Icons\\Ability_DualWield"
							faction = FACTION_BATTLE
							return
						end
					end
				end			
			end
			SetMapZoom(continentIndex, zoneIndex, 1)
		end
		self.obj.icon = "Interface\\Icons\\Achievement_zone_tolbarad"
		faction = FACTION_UNKNOWN	
	end
end

-- Return wait time in "h:m:s", or "Battle" if match in progress
function Broker_TolBarad:GetTimeText(seconds)
	local timeText = ""
	local h = math.floor(seconds / 3600)
	local m = math.floor((seconds % 3600) / 60)
	local s = seconds % 60
	if profileDB.showSeconds then
		if h > 9 then
			timeText = h .. ":"
		elseif h > 0 then
			timeText = "0" .. h .. ":"
		else
			timeText = "00" .. ":"
		end
		if m > 9 then
			timeText = timeText .. m .. ":"
		elseif m > 0 then
			timeText = timeText .. "0" .. m .. ":"
		else
			timeText = timeText .. "00" .. ":"
		end
		if s > 9 then
			timeText = timeText .. s
		elseif s > 0 then
			timeText = timeText .. "0" .. s
		else
			timeText = timeText .. "00"
		end
	elseif seconds > 59 then
		if h > 9 then
			timeText = h .. ":"
		elseif h > 0 then
			timeText = "0" .. h .. ":"
		else
			timeText = "00" .. ":"
		end
		if m > 9 then
			timeText = timeText .. m
		elseif m > 0 then
			timeText = timeText .. "0" .. m
		else
			timeText = timeText .. "00"
		end
	else
		if m > 9 then
			timeText = timeText .. m .. ":"
		elseif m > 0 then
			timeText = timeText .. "0" .. m .. ":"
		else
			timeText = timeText .. "00" .. ":"
		end
		if s > 9 then
			timeText = timeText .. s
		elseif s > 0 then
			timeText = timeText .. "0" .. s
		else
			timeText = timeText .. "00"
		end
	end
	return timeText
end

function Broker_TolBarad:GetAnnounceText(faction, nextBattle)
	local message = "[Broker_TolBarad] "
	if faction == "Horde" or faction == "Alliance" then
		if faction == UnitFactionGroup("player") then
			message = message .. " " .. L["Defensive battle in"]
		else
			message = message .. " " .. L["Offensive battle in"]
		end
	elseif faction == "battle" then
		message = message .. " " .. L["The battle for Tol Barad is underway."]
	else
		message = message .. " " .. L["Next battle in"]
	end
	if not (faction == "battle") then
		if nextBattle then
			local seconds = nextBattle - time()
			local timeText = self:GetTimeText(seconds)
			message = message .. " " .. timeText
		else
			message = message .. " " .. L["Unknown"]
		end
	end
	return message
end

function Broker_TolBarad:Warn(seconds)
	local numGuild, numParty, numRaid = GetNumGuildMembers(), GetNumPartyMembers(), GetNumRaidMembers()
	local msg = "[Broker_TolBarad] "
	
	if seconds == 0 then
		msg = msg .. L["The battle for Tol Barad has begun."]
	else
		if faction == "Horde" or faction == "Alliance" then
			if faction == UnitFactionGroup("player") then
				msg = msg .. L["Defensive battle in"] .. " "
			else
				msg = msg .. L["Offensive battle in"] .. " "
			end
		else
			msg = msg .. " " .. L["Next battle in"] .. " "
		end
		msg = msg .. seconds .. L[" minutes."]
	end
	
	if profileDB.tellSelf then
		DEFAULT_CHAT_FRAME:AddMessage("|cffff0000" .. msg .. "|r")
	end
	if profileDB.tellGuild and (numGuild > 0) then
		SendChatMessage(msg,"GUILD",nil,nil)
	end
	if profileDB.tellParty and (numParty > 0) then
		SendChatMessage(msg,"PARTY",nil,nil)
	end
	if profileDB.tellRaid and (numRaid > 0) then
		SendChatMessage(msg,"RAID",nil,nil)
	end
end
